/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.base.extensions

import com.alibaba.fastjson.PropertyNamingStrategy
import com.sybotan.base.utils.SJsonUtil
import java.lang.Exception
import java.lang.reflect.ParameterizedType

/**
 * Any扩展。获得类的泛型类
 *
 * @return  类的泛型类
 */
fun Any.gennericClass(): Class<*>? {
    return try {
        (javaClass.genericSuperclass as ParameterizedType).actualTypeArguments[0] as Class<*>
    } catch (e: Exception) {
        null
    }
} // Fun Any.gennericClass()

/**
 * Any扩展toJson()函数。
 *
 * @param   namingStrategy          命名规则
 * @return  Json字符串(属性为null不序列化)
 */
fun Any.toJson(namingStrategy: PropertyNamingStrategy? = null): String {
    return SJsonUtil.toJson(this, namingStrategy)
} // Fun Any.toJson()

/**
 * Any扩展toJson()函数。
 *
 * @param   namingStrategy          命名规则
 * @return  Json字符串(属性为null序列化)
 */
fun Any.toJsonAll(namingStrategy: PropertyNamingStrategy? = null): String {
    return SJsonUtil.toJsonAll(this, namingStrategy)
} // Fun Any.toJsonAll()

/**
 * Any扩展toMap()函数。
 *
 * @return  Json字符串
 */
fun Any.toMap(): Map<String, Any?> {
    @Suppress("UNCHECKED_CAST")
    return SJsonUtil.fromJson(this.toJson(), Map::class.java) as Map<String, Any?>
} // Fun Any.toMap()

/**
 * Any扩展toByte()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toBoolean(): Boolean? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> this as Boolean
        "Byte" -> (this as Byte).toBoolean()
        "Double" -> (this as Double).toBoolean()
        "Float" -> (this as Float).toBoolean()
        "Int", "Integer" ->  (this as Int).toBoolean()
        "Long" -> (this as Long).toBoolean()
        "Short" -> (this as Short).toBoolean()
        else -> null
    }
} // Fun Any.toBoolean()

/**
 * Any扩展toByte()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toByte(): Byte? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toByte()
        "Byte" -> this as Byte
        "Double" -> (this as Double).toByte()
        "Float" -> (this as Float).toByte()
        "Int", "Integer" ->  (this as Int).toByte()
        "Long" -> (this as Long).toByte()
        "Short" -> (this as Short).toByte()
        else -> null
    }
} // Fun Any.toByte()

/**
 * Any扩展toDouble()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toDouble(): Double? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toDouble()
        "Byte" -> (this as Byte).toDouble()
        "Double" -> this as Double
        "Float" -> (this as Float).toDouble()
        "Int", "Integer" ->  (this as Int).toDouble()
        "Long" -> (this as Long).toDouble()
        "Short" -> (this as Short).toDouble()
        else -> null
    }
} // Fun Any.toDouble()

/**
 * Any扩展toFloat()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toFloat(): Float? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toFloat()
        "Byte" -> (this as Byte).toFloat()
        "Double" -> (this as Double).toFloat()
        "Float" -> this as Float
        "Int", "Integer" ->  (this as Int).toFloat()
        "Long" -> (this as Long).toFloat()
        "Short" -> (this as Short).toFloat()
        else -> null
    }
} // Fun Any.toFloat()

/**
 * Any扩展toInt()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toInt(): Int? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toInt()
        "Byte" -> (this as Byte).toInt()
        "Double" -> (this as Double).toInt()
        "Float" -> (this as Float).toInt()
        "Int", "Integer" ->  this as Int
        "Long" -> (this as Long).toInt()
        "Short" -> (this as Short).toInt()
        "String" -> {
            try {
                Integer.parseInt(this as String)
            }catch (e: Exception) {
                0
            }
        }
        else -> null
    }
} // Fun Any.toInt()

/**
 * Any扩展toLong()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toLong(): Long? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toLong()
        "Byte" -> (this as Byte).toLong()
        "Double" -> (this as Double).toLong()
        "Float" -> (this as Float).toLong()
        "Int", "Integer" ->  (this as Int).toLong()
        "Long" -> this as Long
        "Short" -> (this as Short).toLong()
        else -> null
    }
} // Fun Any.toLong()

/**
 * Any扩展toShort()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toShort(): Short? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toShort()
        "Byte" -> (this as Byte).toShort()
        "Double" -> (this as Double). toShort()
        "Float" -> (this as Float).toShort()
        "Int", "Integer" ->  (this as Int).toShort()
        "Long" -> (this as Long).toShort()
        "Short" -> this as Short
        else -> null
    }
} // Fun Any.toShort()

/**
 * Any扩展，判断是否为基本类型
 *
 * @return  如果是则返回true
 */
fun Any.isBaseType(): Boolean {
    if (this.javaClass.isPrimitive) {
        return true;
    }

    return  this is String ||
            this is Boolean ||
            this is Char ||
            this is Byte ||
            this is Short ||
            this is Int ||
            this is Long ||
            this is Float ||
            this is Double ||
            this is java.sql.Date ||
            this is java.sql.Time ||
            this is java.sql.Timestamp ||
            this is java.util.Date
} // Fun Any.isBaseType()
