/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.base.extensions

import com.alibaba.fastjson.annotation.JSONField
import java.lang.reflect.Field
import java.lang.reflect.Method

/**
 * Class<*>扩展。获得所有包含父对象的定义字段列表。
 *
 * @return  字段列表
 */
fun Class<*>.getAllDeclaredFields(): ArrayList<Field> {
    val fieldList = ArrayList<Field>()
    var clz: Class<*>? = this
    while (clz != null) {
        fieldList.addAll(clz.declaredFields.toList())
        clz = clz.superclass
    }
    return fieldList
} // Fun Class<*>.getAllDeclaredFields()

/**
 * Class<*>扩展。根据名称获得所有包含父对象的定义字段。
 *
 * @param   ignoreCase      是否忽略大小写，默认不忽略
 * @param   jsonProperty    是否检测Json属性
 * @return  字段
 */
fun Class<*>.getAllDeclaredField(name: String, ignoreCase: Boolean = false, jsonProperty: Boolean = false): Field? {
    val list = this.getAllDeclaredFields()
    return list.first {
        val annName = it.getAnnotation(JSONField::class.java)?.name
        it.name.equals(name, ignoreCase) || (jsonProperty && name.equals(annName, ignoreCase))
    }
} // Fun Class<*>.getAllDeclaredField()

/**
 * Class<*>扩展。获得所有包含父对象的定义方法列表。
 *
 * @return  方法列表
 */
fun Class<*>.getAllDeclaredMethods(): ArrayList<Method> {
    val methodList = ArrayList<Method>()
    var clz: Class<*>? = this
    while (clz != null) {
        methodList.addAll(clz.declaredMethods.toList())
        clz = clz.superclass
    }
    return methodList
} // Fun Class<*>.getAllDeclaredMethods()

/**
 * Class<*>扩展，判断是否为基本类型
 *
 * @return  如果是则返回true
 */
fun Class<*>.isBaseType(): Boolean {
    if (this.isPrimitive) {
        return true;
    }

    return  String::class.java.isAssignableFrom(this) ||
            Boolean::class.java.isAssignableFrom(this) ||
            Char::class.java.isAssignableFrom(this) ||
            Byte::class.java.isAssignableFrom(this) ||
            Short::class.java.isAssignableFrom(this) ||
            Int::class.java.isAssignableFrom(this) ||
            Long::class.java.isAssignableFrom(this) ||
            Float::class.java.isAssignableFrom(this) ||
            Double::class.java.isAssignableFrom(this) ||
            java.sql.Date::class.java.isAssignableFrom(this) ||
            java.sql.Time::class.java.isAssignableFrom(this) ||
            java.sql.Timestamp::class.java.isAssignableFrom(this) ||
            java.util.Date::class.java.isAssignableFrom(this)

} // Fun Class<*>.isBaseType()