/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.base.utils

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * Gzip工具，用于完成zip压缩与解压缩
 *
 * @author  庞利祥 <sybotan@126.com>
 */
object SGzipUtil {
    const val GZIP_ENCODE_UTF_8 = "UTF-8"
    const val GZIP_ENCODE_ISO_8859_1 = "ISO-8859-1"

    /**
     * 字符串的解压
     *
     * @param   str         对字符串解压
     * @return  返回解压缩后的字符串
     */
    @Throws(IOException::class)
    fun unCompress(str: String): ByteArray? {
        if (str.isEmpty()) {
            return null
        }

        // 创建一个新的 byte 数组输出流
        val out = ByteArrayOutputStream()
        // 创建一个 ByteArrayInputStream，使用 buf 作为其缓冲区数组
        val `in` = ByteArrayInputStream(str
                .toByteArray(charset("UTF-8")))
        // 使用默认缓冲区大小创建新的输入流
        val gzip = GZIPInputStream(`in`)
        val buffer = ByteArray(1024)
        var n = gzip.read(buffer)
        while (n >= 0) {  // 将未压缩数据读入字节数组
            // 将指定 byte 数组中从偏移量 off 开始的 len 个字节写入此 byte数组输出流
            out.write(buffer, 0, n)
            n = gzip.read(buffer)
        }
        // 使用指定的 charsetName，通过解码字节将缓冲区内容转换为字符串
        return buffer
    } // Fun unCompress()

    /**
     * 字符串压缩为GZIP字节数组
     */
    fun compress(str: String, encoding: String): ByteArray? {
        if (str.isEmpty()) {
            return null
        }

        val out = ByteArrayOutputStream()
        val gzip: GZIPOutputStream
        try {
            gzip = GZIPOutputStream(out)
            gzip.write(str.toByteArray(charset(encoding)))
            gzip.close()
        } catch (e: IOException) {
            println("gzip compress error: " + e.message)
        }

        return out.toByteArray()
    } // Fun compress()

    /**
     * Gzip  byte[] 解压成字符串
     */
    fun uncompressToString(bytes: ByteArray): String? {
        return uncompressToString(bytes, GZIP_ENCODE_UTF_8)
    } // Fun uncompressToString()


    /**
     * Gzip  byte[] 解压成字符串
     */
    private fun uncompressToString(bytes: ByteArray?, encoding: String): String? {
        if (bytes == null || bytes.size == 0) {
            return null
        }
        val out = ByteArrayOutputStream()
        val `in` = ByteArrayInputStream(bytes)
        try {
            val ungzip = GZIPInputStream(`in`)
            val buffer = ByteArray(1024)
            var n = ungzip.read(buffer)
            while (n >= 0) {
                out.write(buffer, 0, n)
                n = ungzip.read(buffer)
            }
            return out.toString(encoding)
        } catch (e: IOException) {
            println("gzip compress error:" + e.message)
        }

        return null
    } // Fun uncompressToString()

//    /**
//     * 判断byte[]是否是Gzip格式
//     */
//    fun isGzip(data: ByteArray): Boolean {
//        val header = data[0] shl 8 or (data[1] and 0xff)
//        return header == 0x1f8b
//    } // Fun isGzip()

//    /**
//     * GZIP解压缩
//     *
//     * @param   zipText     GZIP压缩过的字符串
//     * @return 解压后的字符串
//     */
//    @Throws(IOException::class)
//    fun decompress(zipText: String): String {
//        val compressed = Base64.decode(zipText, 1024)
//        if (compressed.size > 4) {
//            val gzipInputStream = GZIPInputStream(
//                    ByteArrayInputStream(compressed, 4,
//                            compressed.size - 4))
//
//            val baos = ByteArrayOutputStream()
//            var value = 0
//            while (value != -1) {
//                value = gzipInputStream.read()
//                if (value != -1) {
//                    baos.write(value)
//                }
//            }
//            gzipInputStream.close()
//            baos.close()
//            return String(baos.toByteArray(), "UTF-8")
//        } else {
//            return ""
//        }
//    }
} // Object GzipUtil