/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.database

import com.sybotan.database.enums.SOps

/**
 * 过滤条件
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SFilter private constructor(
        /** 第一参数，保存属性名 */
        var first: Any?,
        /** 第一参数，保存值 */
        var second: Any?,
        /** 操作符 */
        var opt: SOps = SOps.EQ) {
    constructor(): this(null, null)

    companion object {
        /**
         * 过滤条件：属性 == 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun eq(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.EQ)
        } // Fun eq()

        /**
         * 过滤条件：函数 == 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun eq(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.EQ)
        } // Fun eq()

        /**
         * 过滤条件：属性 != 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun ne(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.NE)
        } // Fun ne()

        /**
         * 过滤条件：函数 != 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun ne(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.NE)
        } // Fun ne()

        /**
         * 过滤条件：属性 > 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun gt(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.GT)
        } // Fun gt()

        /**
         * 过滤条件：函数 > 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun gt(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.GT)
        } // Fun gt()

        /**
         * 过滤条件：属性 >= 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun gte(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.GTE)
        } // Fun gte()

        /**
         * 过滤条件：函数 >= 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun gte(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.GTE)
        } // Fun gte()

        /**
         * 过滤条件：属性 < 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun lt(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.LT)
        } // Fun lt()

        /**
         * 过滤条件：函数 < 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun lt(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.LT)
        } // Fun lt()

        /**
         * 过滤条件：属性 <= 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun lte(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.LTE)
        } // Fun lte()

        /**
         * 过滤条件：函数 <= 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun lte(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.LTE)
        } // Fun lte()

        /**
         * 过滤条件：包含
         *
         * @param   property        属性名
         * @param   value           被包含的值
         * @return  过滤条件
         */
        fun contain(property: String, value: String): SFilter {
            return SFilter(property, value, SOps.CONTAIN)
        } // Fun contain()

        /**
         * 过滤条件：包含
         *
         * @param   func            函数对象
         * @param   value           被包含的值
         * @return  过滤条件
         */
        fun contain(func: SAbstractFunction, value: String): SFilter {
            return SFilter(func, value, SOps.CONTAIN)
        } // Fun contain()

        /**
         * 过滤条件：以指定字符串开始
         *
         * @param   property        属性名
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun startWith(property: String, value: String): SFilter {
            return SFilter(property, value, SOps.STARTWITH)
        } // Fun contain()

        /**
         * 过滤条件：以指定字符串开始
         *
         * @param   func            函数对象
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun startWith(func: SAbstractFunction, value: String): SFilter {
            return SFilter(func, value, SOps.STARTWITH)
        } // Fun contain()

        /**
         * 过滤条件：以指定字符串结尾
         *
         * @param   property        属性名
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun endWith(property: String, value: String): SFilter {
            return SFilter(property, value, SOps.ENDWITH)
        } // Fun endWith()

        /**
         * 过滤条件：以指定字符串结尾
         *
         * @param   func            函数对象
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun endWith(func: SAbstractFunction, value: String): SFilter {
            return SFilter(func, value, SOps.ENDWITH)
        } // Fun endWith()

        /**
         * 过滤条件：属性为空
         *
         * @param   property        属性名
         * @return  过滤条件
         */
        fun isNull(property: String): SFilter {
            return SFilter(property, null, SOps.ISNULL)
        } // Fun isNull()

        /**
         * 过滤条件：属性为空
         *
         * @param   func            函数对象
         * @return  过滤条件
         */
        fun isNull(func: SAbstractFunction): SFilter {
            return SFilter(func, null, SOps.ISNULL)
        } // Fun isNull()

        /**
         * 过滤条件：属性在值列表中
         *
         * @param   property        属性名
         * @param   valueList       值列表
         * @return  过滤条件
         */
        fun inList(property: String, valueList: List<Any>): SFilter {
            return SFilter(property, valueList, SOps.IN)
        } // Fun inList()

        /**
         * 过滤条件：属性在值列表中
         *
         * @param   func            函数对象
         * @param   valueList       值列表
         * @return  过滤条件
         */
        fun inList(func: SAbstractFunction, valueList: List<Any>): SFilter {
            return SFilter(func, valueList, SOps.IN)
        } // Fun inList()

        /**
         * 过滤条件：逻辑非
         *
         * @param   filter          属性名
         * @return  过滤条件
         */
        fun not(filter: SFilter): SFilter {
            return SFilter(filter, null, SOps.NOT)
        } // Fun not()

        /**
         * 过滤条件：逻辑与
         *
         * @param   filterList      过滤条件列表
         * @return  过滤条件
         */
        fun and(vararg filterList: SFilter): SFilter {
            var filter = filterList[0]
            for (i in 1 until filterList.size) {
                filter = SFilter(filter, filterList[i], SOps.AND)
            }
            return filter
        } // Fun not()

        /**
         * 过滤条件：逻辑或
         *
         * @param   filterList      过滤条件列表
         * @return  过滤条件
         */
        fun or(vararg filterList: SFilter): SFilter {
            var filter = filterList[0]
            for (i in 1 until filterList.size) {
                filter = SFilter(filter, filterList[i], SOps.OR)
            }
            return filter
        } // Fun not()
    } // companion object
} // Class SFilter