/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.database

import com.sybotan.base.exceptions.SFieldNotFoundException
import com.sybotan.base.extensions.getAllDeclaredField
import com.sybotan.base.extensions.toJson
import com.sybotan.base.opts.SFormat
import com.sybotan.database.enums.SDirection
import com.sybotan.database.parser.SSyntaxParser
import java.io.ByteArrayInputStream
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

/**
 * 查询条件生成器
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SQueryBuilder<ENTITY: Any>(
        private val baseDao: SAbstractDao<ENTITY>,
        val filterList: ArrayList<SFilter>) {

    init {
        // 转换数据类型
        for (item in filterList) {
            convertType(baseDao.entityClass, item)
        }
    } // Init

    /** 表名 */
    var tableName: String? = null
    /** 映射字段列表 */
    var fieldList = ArrayList<String>()
    /** 排序条件 */
    var orderArgs = ArrayList<SOrder>()
    /** 是否去除重复记录 */
    var isDistinct = false
    /** 偏移量 */
    var offset = 0
        set(value) {
            field = value
            if (field < 0) {
                field = 0
            }
        }

    /** 记录数 */
    var count:Int? = null
//        set(value) {
//            field = value
//            if (field < 1) {
//                field = 1
//            }
//        }

    /** 分组 */
    var group = SGroup()

    /**
     * 执行查询
     *
     * @return  返回查询到的记录
     */
    fun exec(): ArrayList<ENTITY> {
        return baseDao.execQuery(this)
    } // Fun exec()

    /**
     * 统计查询个数
     *
     * @return  返回查询到的记录个数
     */
    fun execCount(): Long {
        return baseDao.execCount(this)
    } // Fun execCount()

    /**
     * 执行查询，并返回第一条记录
     *
     * @return  返回查询到的第一条记录
     */
    fun entity(): ENTITY? {
        count = 1
        val list = exec()
        return if (list.size > 0) {
            list[0]
        } else {
            null
        }
    } // Fun entity()

    /**
     * 执行分页查询
     *
     * @param   pageNumber      第几页
     * @param   pageSize        每页记录数
     * @return  分页找到的数据
     */
    fun pageQuery(pageNumber: Int, pageSize: Int): SPageQueryInfo<ENTITY> {
        return baseDao.pageQuery(this, pageNumber, pageSize)
    } // Fun pageQuery()

    /**
     * 数据库表
     *
     * @param   name    表名
     * @return  查询构建器
     */
    fun table(name: String): SQueryBuilder<ENTITY> {
        tableName = name
        return this
    } // Fun tableName

    /**
     * 字段映射
     *
     * @param   fields      字段列表
     * @return  查询构建器
     */
    fun projection(fields: List<String>): SQueryBuilder<ENTITY> {
        for (name in fields) {
            fieldList.add(propertyName(name.trim()))
        }
        return this
    } // Fun projection()

    /**
     * 字段映射
     *
     * @param   fields      字段列表
     * @return  查询构建器
     */
    fun projection(fields: String): SQueryBuilder<ENTITY> {
        return projection(fields.split(","))
    } // Fun projection()

    /**
     * 添加过滤条件
     *
     * @param   filterList      过滤条件
     * @return  查询构建器
     */
    fun filter(vararg filterList: SFilter): SQueryBuilder<ENTITY> {
        for (item in filterList) {
            convertType(baseDao.entityClass, item)
            this.filterList.add(item)
        }
        return this
    } // Fun filter()

    /**
     * 添加过滤条件
     *
     * @param   filterList      过滤条件
     * @return  查询构建器
     */
    fun filter(vararg filterList: Pair<String, Any>): SQueryBuilder<ENTITY> {
        for (item in filterList) {
            val filter = SFilter.eq(item.first, item.second)
            convertType(baseDao.entityClass, filter)
            this.filterList.add(filter)
        }
        return this
    } // Fun filter()

    /**
     * 添加排序条件
     *
     * @param   name    排序属性名
     * @param   dir     排序方向
     * @return  查询构建器
     */
    fun order(name: String, dir: SDirection): SQueryBuilder<ENTITY> {
        orderArgs.add(SOrder(propertyName(name.trim()), dir))
        return this
    } // Fun order()

    /**
     * 添加排序条件
     *
     * @param   orderList   排序条件
     * @return  查询构建器
     */
    fun order(vararg orderList: SOrder): SQueryBuilder<ENTITY> {
        for (order in orderList) {
            order(order.name, order.direction)
        }
        return this
    } // Fun order()

    /**
     * 添加排序条件
     *
     * @param   orderStr    排序条件
     * @return  查询构建器
     */
    fun order(orderStr: String): SQueryBuilder<ENTITY> {
        val parser = SSyntaxParser(ByteArrayInputStream(orderStr.toByteArray()))
        val orderList = parser.parserOrders()
        order(*orderList.toTypedArray())
        return this
    } // Fun order()

    /**
     * 去除重复记录
     *
     * @return  查询构建器
     */
    fun distinct(): SQueryBuilder<ENTITY> {
        isDistinct = true
        return this
    } // Fun distinct()

    /**
     * 限定取得的记录数
     *
     * @param   count   取得的记录数
     * @return  查询构建器
     */
    fun limit(count: Int): SQueryBuilder<ENTITY> {
        limit(0, count)
        return this
    } // Fun limit()

    /**
     * 限定从指定位置取得的记录数
     *
     * @param   offset  指定位置(默认从0开始)
     * @param   count   取得的记录数
     * @return  查询构建器
     */
    fun limit(offset: Int, count: Int): SQueryBuilder<ENTITY> {
        this.offset = offset
        this.count = count
        return this
    } // Fun limit()

    /**
     * 添加分组
     *
     * @param   nameList    名称列表
     * @return  查询构建器
     */
    fun group(vararg nameList: String): SQueryBuilder<ENTITY> {
        for (name in nameList) {
            group.nameList.add(propertyName(name.trim()))
        }
        return this
    } // Fun group()

    /**
     * 是否统计记录个数
     */
    fun count(value: Boolean): SQueryBuilder<ENTITY>{
        group.count = value
        return this
    } // Fun count()
    /**
     * 统计最大数
     *
     * @param   nameList    统计的属性名称
     * @return  查询构建器
     */
    fun max(vararg nameList: String): SQueryBuilder<ENTITY> {
        for (name in nameList) {
            group.maxList.add(propertyName(name.trim()))
        }
        return this
    } // Fun max()


    /**
     * 统计最小数
     *
     * @param   nameList    统计的属性名称
     * @return  查询构建器
     */
    fun min(vararg nameList: String): SQueryBuilder<ENTITY> {
        for (name in nameList) {
            group.minList.add(propertyName(name.trim()))
        }
        return this
    } // Fun min()

    /**
     * 统计求和
     *
     * @param   nameList    统计的属性名称
     * @return  查询构建器
     */
    fun sum(vararg nameList: String): SQueryBuilder<ENTITY> {
        for (name in nameList) {
            group.sumList.add(propertyName(name.trim()))
        }
        return this
    } // Fun sum()

    /**
     * 统计求平均数
     *
     * @param   nameList    统计的属性名称
     * @return  查询构建器
     */
    fun avg(vararg nameList: String): SQueryBuilder<ENTITY> {
        for (name in nameList) {
            group.avgList.add(propertyName(name.trim()))
        }
        return this
    } // Fun avg()

    /**
     * 获得属性名
     *
     * @param   name    输入名称
     * @return  属性名
     */
    private fun propertyName(name: String): String {
        try {
            val clz = baseDao.entityClass
            return clz.getAllDeclaredField(name, ignoreCase = true, jsonProperty = true)?.name!!
        } catch (e: Exception) {
            e.printStackTrace()
            throw SFieldNotFoundException(name)
        }
    } // Fun propertyName()

    /**
     * 转换数据类型
     *
     * @param   entityClass 实体类类型
     * @param   filter      过滤条件
     */
    private fun convertType(entityClass: Class<*>, filter: SFilter) {
        if (filter.first is SFilter) {
            convertType(entityClass, filter.first as SFilter)
        }
        if (filter.second is SFilter) {
            convertType(entityClass, filter.second as SFilter)
        }

        if (filter.first is String) {
            val name = filter.first as String
            if (name.indexOf(".") < 0) {
                val field = entityClass.getAllDeclaredField(filter.first as String, ignoreCase = true, jsonProperty = true)
                        ?: return
                // 如果数据类型为日期型
                if (Date::class.java.isAssignableFrom(field.type)) {
                    val ft = SimpleDateFormat(SFormat.dateTime)
                    filter.second = ft.parse(filter.second as String)
                }
            }
        }
    } // Fun convertType()
} // Class SQueryBuilder
