/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.database.test.parser

import org.testng.annotations.DataProvider
import org.testng.annotations.Test

/**
 * 语法解析器-测试类
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SSyntaxParserTest {

    /**
     * 提供正确过滤条件测试数据
     *
     * @param   filter      过滤条件
     * @param   result      预期结果
     */
    @DataProvider(name="filter")
    fun goodFilterDataProvider(): Array<Array<String>> {
        return arrayOf(
                // 关系
                arrayOf("id = 1", "[{\"first\":\"id\",\"second\":1,\"opt\":\"EQ\"}]"),
                arrayOf("id = '2'", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"EQ\"}]"),
                arrayOf("id > '2'", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"GT\"}]"),
                arrayOf(" id >= '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"GTE\"}]"),
                arrayOf(" id < '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"LT\"}]"),
                arrayOf(" id <= '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"LTE\"}]"),
                arrayOf(" id != '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"NE\"}]"),
                // 包含
                arrayOf(" id CONTAIN '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                arrayOf(" id contain '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                arrayOf(" id ContaiN '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                arrayOf(" id CoNTaIN '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                // 以XXX开始
                arrayOf(" id STARTWITH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"STARTWITH\"}]"),
                arrayOf(" id startwith '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"STARTWITH\"}]"),
                arrayOf(" id STarTWiTH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"STARTWITH\"}]"),
                // 以XXX结束
                arrayOf(" id ENDWITH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"ENDWITH\"}]"),
                arrayOf(" id endwith '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"ENDWITH\"}]"),
                arrayOf(" id EndWiTH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"ENDWITH\"}]"),
                // 为空
                arrayOf(" id ISNULL ", "[{\"first\":\"id\",\"opt\":\"ISNULL\"}]"),
                arrayOf(" id isnull ", "[{\"first\":\"id\",\"opt\":\"ISNULL\"}]"),
                arrayOf(" id IsNull ", "[{\"first\":\"id\",\"opt\":\"ISNULL\"}]"),
                // 不为空
                arrayOf(" id NOTNULL ", "[{\"first\":\"id\",\"opt\":\"NOTNULL\"}]"),
                arrayOf(" id notnull ", "[{\"first\":\"id\",\"opt\":\"NOTNULL\"}]"),
                arrayOf(" id NotNULL ", "[{\"first\":\"id\",\"opt\":\"NOTNULL\"}]"),
                // 数据类型
                arrayOf(" used = true", "[{\"first\":\"used\",\"second\":true,\"opt\":\"EQ\"}]"),
                arrayOf(" used != TRUE", "[{\"first\":\"used\",\"second\":true,\"opt\":\"NE\"}]"),
                arrayOf(" used = TruE", "[{\"first\":\"used\",\"second\":true,\"opt\":\"EQ\"}]"),
                arrayOf(" used = false", "[{\"first\":\"used\",\"second\":false,\"opt\":\"EQ\"}]"),
                arrayOf(" used != FALSE", "[{\"first\":\"used\",\"second\":false,\"opt\":\"NE\"}]"),
                arrayOf(" used = FaLse", "[{\"first\":\"used\",\"second\":false,\"opt\":\"EQ\"}]"),
                arrayOf(" age = 18", "[{\"first\":\"age\",\"second\":18,\"opt\":\"EQ\"}]"),
                arrayOf(" width = 18.15", "[{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"}]"),
                // 括号表达式
                arrayOf(" (  width = 18.15 )", "[{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"}]"),
                // NOT
                arrayOf(" not width = 18.15", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" NOT width = 18.15", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" Not width = 18.15", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" !width = 18.15 ", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" !(width = 18.15)", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                // OR
                arrayOf(" (id = '1') or (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') OR (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') || (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') or (width>=2) || height > 3", "[{\"first\":{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"OR\"}]"),
                // AND
                arrayOf(" (id = '1') and (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"}]"),
                arrayOf(" (id = '1') AND (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"}]"),
                arrayOf(" (id = '1') && (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"}]"),
                arrayOf(" (id = '1') anD (width>=2) && height > 3", "[{\"first\":{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"AND\"}]"),
                // 逻辑混合
                arrayOf(" (id = '1') and (width>=2) || height > 3", "[{\"first\":{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') || (width>=2) && height > 3", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"AND\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') and (width>=2 || height > 3)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"OR\"},\"opt\":\"AND\"}]"),
                // 条件列表
                arrayOf(" width <= 18.15; age>18; name='abc'", "[{\"first\":\"width\",\"second\":18.15,\"opt\":\"LTE\"},{\"first\":\"age\",\"second\":18,\"opt\":\"GT\"},{\"first\":\"name\",\"second\":\"abc\",\"opt\":\"EQ\"}]")
        )
    } // Fun goodFilterDataProvider()

    /**
     * 正确过滤条件测试
     *
     * @param   filter      过滤条件
     * @param   expected    预期结果
     */
    @Test(dataProvider="filter")
    fun goodFilterTest(filter: String, expected: String) {
//        val queryParser = SSyntaxParser(ByteArrayInputStream(filter.toByteArray()))
//        val filters  = queryParser.parserFilters()
//        Assert.assertEquals(filters.toJson(), expected)
        return
    } // Fun goodFilterTest()

} // Class SSyntaxParserTest
