/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.mybatis

import com.alibaba.fastjson.PropertyNamingStrategy
import com.sybotan.base.extensions.*
import com.sybotan.base.utils.SJsonUtil
import org.slf4j.LoggerFactory
import javax.persistence.Column

/**
 * 记录解析器
 *
 * @author  庞利祥(庞利祥)
 */
open class SRowParser<ENTITY: Any>(entityClazz: Class<ENTITY>) {
    companion object {
        /** 日志记录器 */
        private val logger = LoggerFactory.getLogger(SRowParser::class.java)
    }
    /** 实体类类型 */
    private val entityClass = entityClazz

    /**
     * 解析记录行
     *
     * @param   columns     字段映射
     * @return  实体类
     */
    open fun parseRow(columns: Map<String, Any?>): ENTITY {
        val entity = entityClass.getDeclaredConstructor().newInstance()
        for (field in entityClass.getAllDeclaredFields()) {
            val accessible = field.isAccessible    // canAccess(entity)
            field.isAccessible = true

            // 处理统计信息
            if (field.name == "statistics") {
                field.set(entity, groupStatistics(columns))
                continue
            }
            val anno = field.getAnnotation(Column::class.java) ?: continue

            val value = columns[anno.name] ?: continue
            try {
                when {
                    field.type.isBaseType() -> {
                        field.set(entity, value)
                    }
                    Map::class.java.isAssignableFrom(field.type) -> {
                        field.set(entity, SJsonUtil.fromJson(value.toString(), HashMap::class.java))
                    }
                    List::class.java.isAssignableFrom(field.type) -> {
                        try {
                            val genericType = field.genericType
                            val v1 = SJsonUtil.fromJson(value.toString(), genericType!!)
                            field.set(entity, v1)
                        } catch (e: Exception) {
                            // DO NOTHING
                        }
                    }
                    else -> {
                        // field.set(entity, SJsonUtil.fromJson(value.toString(), field.type))
                        field.set(entity, SJsonUtil.fromJson(value.toString(), field.type))
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            field.isAccessible = accessible
        }

        return entity as ENTITY
    } // Fun parseRow()

    /**
     * 获得分组统计信息
     *
     * @param   columns     字段列表
     * @return  分组统计信息
     */
    private fun groupStatistics(columns: Map<String, Any?>): HashMap<String, Any?>? {
        var statistics: HashMap<String, Any?>?
        statistics = HashMap()
        for (col in columns) {
            if (!col.key.startsWith("_statistics_")) {
                continue
            }
            if (null == statistics) {
                statistics = HashMap()
            }

            when {
                col.key.startsWith("_statistics_count") ->
                    statistics[statisticName("Count", "")] = col.value
                col.key.startsWith("_statistics_avg_") ->
                    statistics[statisticName("Avg", col.key)] = col.value
                col.key.startsWith("_statistics_sum_") ->
                    statistics[statisticName("Sum", col.key)] = col.value
                col.key.startsWith("_statistics_max_") ->
                    statistics[statisticName("Max", col.key)] = col.value
                col.key.startsWith("_statistics_min_") ->
                    statistics[statisticName("Min", col.key)] = col.value
            }
        }

        return statistics
    } // Fun groupStatistics()

    /**
     * 统记字段名称
     *
     * @param   type        统计类型
     * @param   name        统计字段名
     * @return  字段名
     */
    private fun statisticName(type: String, name: String): String {
        val col = if (name.startsWith("_statistics_")) {
            name.substring(16)
        } else {
            ""
        }

        val str = "$col$type"
        return if (PropertyNamingStrategy.PascalCase == SJsonUtil.serializeConfig.propertyNamingStrategy) {
            str.firstUpperCase();
        } else {
            str.firstLowerCase();
        }
    } // Fun statisticName()
} // SRowParser