/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.mybatis

import org.apache.ibatis.jdbc.SQL
import org.slf4j.LoggerFactory
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

/**
 * SQL构造器
 *
 * @author  庞利祥 <sybotan@126.com>
 */
open class SSqlProvider {
    /** 静态对象 */
    companion object {
        /** 日志记录器 */
        private val logger = LoggerFactory.getLogger(SSqlProvider::class.java)

        /** 驱动 */
        lateinit var driver: SAbstractDriver;
    } // companion object

    /**
     * 在数据库中插入实体
     *
     * @param   argsMap         注入参数
     * @return  插入SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun insert(argsMap: Map<String, Any?>): String {
        return driver.insert(argsMap)
    } // Fun insert()

    /**
     * 在数据库中替换实体
     *
     * @param   argsMap         注入参数
     * @return  替换SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun replace(argsMap: Map<String, Any?>): String {
        val insertSql = insert(argsMap)
        return driver.replace(argsMap, insertSql)
    } // Fun replace()

    /**
     * 根据条件删除
     *
     * @param   argsMap         注入参数
     * @return  删除SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun delete(argsMap: Map<String, Any?>): String {
        return driver.delete(argsMap)
    } // Fun delete()

    /**
     * 删除所有记录
     *
     * @return  删除SQL语句
     */
    open fun deleteAll(argsMap: Map<String, Any?>): String {
        return driver.deleteAll(argsMap)
    } // Fun deleteAll()

    /**
     * 根据条件更新
     *
     * @param   argsMap         注入参数
     * @return  更新SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun update(argsMap: Map<String, Any?>): String {
        return driver.update(argsMap)
    } // Fun update()

    /**
     * 执行查询操作
     *
     * @param   argsMap         注入参数
     * @return  查询SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    fun execQuery(argsMap: Map<String, Any?>): String {
        return driver.execQuery(argsMap)
    } // Fun execQuery()

    /**
     * 执行查询操作
     *
     * @param   argsMap         注入参数
     * @return  查询SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    fun execCount(argsMap: Map<String, Any?>): String {
        return driver.execCount(argsMap)
    } // Fun execQuery()

//    /**
//     * 构造where子句参数
//     *
//     * @param   filterList      过滤条件
//     * @param   values          子句参数列表
//     * @return  返回where子句
//     */
//    private fun buildWhereArgList(clazz: Class<*>, filterList: List<SFilter>, values: HashMap<String, Any>): String {
//        val builder = StringBuilder()
//
//        for (filter in filterList) {
//            val where = buildWhereArg(clazz, filter, values)
//            if (builder.isNotEmpty()) {
//                builder.append(" AND ")
//            }
//            builder.append(where)
//        }
//
//        val sql = builder.toString()
//        logger.debug("buildWhereArgList = $sql")
//        return sql
//    } // Fun buildWhereArgList()
//
//    /**
//     * 构造where子句参数
//     *
//     * @param   clazz           对应的实体类类型
//     * @param   filter          过滤条件
//     * @param   values          子句参数列表
//     * @return  返回where子句
//     */
//    private fun buildWhereArg(clazz: Class<*>, filter: SFilter, values: HashMap<String, Any>): String {
//        val builder = StringBuilder()
//        val argName = "arg${values.size}"
//        when (filter.opt) {
//            SOps.EQ,            // 等于
//            SOps.NE,            // 不等于
//            SOps.GT,            // 大于
//            SOps.GTE,           // 大于等于
//            SOps.LT,            // 小于
//            SOps.LTE,           // 小于等于
//            SOps.CONTAIN,       // 包含
//            SOps.STARTWITH,     // 以指定字符串开始
//            SOps.ENDWITH,       // 以指定字符串结束
//            SOps.ISNULL         // 为空
//            -> {
//                var path: String? = null
//                var propertyName = if (filter.first is SAbstractFunction) {
//                    (filter.first!! as SAbstractFunction).name
//                } else {
//                    filter.first as String
//                }
//                val pos = propertyName.indexOf(".")
//                if (pos > 0) {
//                    path = propertyName.substring(pos + 1)
//                    propertyName = propertyName.substring(0, pos)
//                }
//                val colName = SAbstractDao.columnName(clazz, propertyName)
//                try {
//                    values[argName] = filter.second!!
//                } catch (e: Exception) {
//                    // DO NOTHING
//                }
//
//                val isJson = if (filter.opt == SOps.CONTAIN || filter.opt == SOps.STARTWITH || filter.opt == SOps.ENDWITH) {
//                    false
//                } else {
//                    path != null
//                }
//
//                return if (filter.first is SAbstractFunction) {
//                    driver.whereArg(driver.processFunction(filter.first!! as SAbstractFunction, driver.argName(colName, path)), driver.argValue(argName, isJson), filter.opt)
//                } else {
//                    driver.whereArg(driver.argName(colName, path), driver.argValue(argName, isJson), filter.opt)
//                }
//            }
//            SOps.IN -> {         // 在列表内
//                // 如果列表不为空，则添加IN子句
//                if (filter.second !=null && List::class.java.isAssignableFrom(filter.second!!.javaClass) &&
//                        (filter.second as List<Any?>).size > 0) {
//                    var path: String? = null
//                    val inArg = buildInArgs(filter, values, argName)
//
//                    if (filter.first is SAbstractFunction) {
//                        val colName = SAbstractDao.columnName(clazz, (filter.first as SAbstractFunction).name)
//                        val col = driver.processFunction(filter.first!! as SAbstractFunction, driver.argName(colName, path))
//                        builder.append("$col IN ($inArg)")
//                    } else {
//                        var propertyName = filter.first!!.toString()
//                        val pos = propertyName.indexOf(".")
//                        if (pos > 0) {
//                            path = propertyName.substring(pos + 1)
//                            propertyName = propertyName.substring(0, pos)
//                        }
//                        val colName = SAbstractDao.columnName(clazz, propertyName)
//                        builder.append(driver.inArg(colName, path, inArg))
//                    }
//                    //"${driver.keyEsc}$colName${driver.keyEsc} IN ($inArg)"
//                }
//            }
//            SOps.NOT -> {        // 逻辑非
//                builder.append("NOT (${buildWhereArg(clazz, filter.first as SFilter, values)})")
//            }
//            SOps.AND -> {        // 逻辑与
//                builder.append("(${buildWhereArg(clazz, filter.first as SFilter, values)}) AND (${buildWhereArg(clazz, filter.second as SFilter, values)})")
//            }
//            SOps.OR -> {         // 逻辑或
//                builder.append("((${buildWhereArg(clazz, filter.first as SFilter, values)}) OR (${buildWhereArg(clazz, filter.second as SFilter, values)}))")
//            }
//            else -> {
//            }
//        }
//
//        return builder.toString()
//    } // Fun buildWhereArgList()
//
//    /**
//     * 生成IN操作参数
//     */
//    @Suppress("UNCHECKED_CAST")
//    private fun buildInArgs(filter: SFilter, values: HashMap<String, Any>, argName: String): String {
//        val builder = StringBuilder()
//        val argList = filter.second!! as List<Any>
//        val property = filter.first?.toString()
//        val isJson = if (filter.first is SAbstractFunction) {
//            false
//        } else {
//            property?.indexOf(".") != -1
//        }
//
//        for ((index, item) in argList.withIndex()) {
//            if (index > 0) {
//                builder.append(", ")
//            }
//            builder.append(driver.argValueToJson("#{whereArgs.${argName}_$index}", isJson))
//            values["${argName}_$index"] = item
//        }
//
//        return builder.toString()
//    } // Fun buildInArgs()
//
//    /**
//     * 构建columns值。如果不指定字段列表，则查所有记录
//     *
//     * @param   clazz       实体类类型
//     * @param   fieldList   属性映射列表
//     * @return  values列表
//     */
//    protected fun buildColumns(clazz: Class<*>, fieldList: ArrayList<String>?): String {
//        // 如果不指定字段列表，则查所有记录
//        if (fieldList == null || fieldList.size <= 0) {
//            return "*"
//        }
//
//        val builder = StringBuilder()
//        for (field in clazz.getAllDeclaredFields()) {
//            if (!fieldList.contains(field.name)) {
//                continue
//            }
//            val anna = field.getAnnotation(Column::class.java) ?: continue
//
//            if (builder.isNotEmpty()) {
//                builder.append(", ")
//            }
//
//            builder.append(driver.escName(anna.name))
//        }
//        return builder.toString()
//    } // Fun buildColumns()

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 级联操作相关
    /**
     * 查询级联关系
     *
     * @param   argsMap         注入参数
     * @return  查询SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    fun relationQuery(argsMap: Map<String, Any?>): String {
        val table = argsMap["table"] as String
        val id1 = argsMap["id1"] as String
        val id2 = argsMap["id2"] as String
        val self = argsMap["self"] as Boolean
        val filter = argsMap["filter"] as String
        val idList = argsMap["idList"] as ArrayList<Any>
        val whereArgs = argsMap["whereArgs"] as HashMap<String, Any>

        var sql = object: SQL() {
            init {
                SELECT("${driver.escName(id1)} as id1, ${driver.escName(id2)} as id2")
                FROM(table)

                val builder = StringBuilder()
                for ((index, item) in idList.withIndex()) {
                    if (index > 0) {
                        builder.append(", ")
                    }
                    builder.append("#{whereArgs.args_$index}")
                    whereArgs["args_$index"] = item
                }
                if (filter != "") {
                    WHERE(filter)
                }
                WHERE("${driver.escName(id1)} IN (${builder.toString()})")
            }
        }.toString()

        // 如果是关联自身（比如楼层连通关系，两个ID都是楼层）
        if (self) {
            val sql1 = object: SQL() {
                init {
                    SELECT("${driver.escName(id2)} as id1, ${driver.escName(id1)} as id2")
                    FROM(table)

                    val builder = StringBuilder()
                    for ((index, item) in idList.withIndex()) {
                        if (index > 0) {
                            builder.append(", ")
                        }
                        builder.append("#{whereArgs.args_$index}")
                        whereArgs["args_$index"] = item
                    }
                    if (filter != "") {
                        WHERE(filter)
                    }
                    WHERE("${driver.escName(id2)} IN (${builder.toString()})")
                }
            }.toString()

            sql = "$sql \n UNION \n $sql1"
        }

        logger.debug("SQL= $sql")
        return sql
    } // Fun relationQuery()
} // Class SSqlProvider
