/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.mybatis.extensions

import com.sybotan.base.extensions.getAllDeclaredFields
import com.sybotan.base.extensions.isBaseType
import com.sybotan.base.extensions.toJson
import java.lang.reflect.Field
import javax.persistence.Column

/**
 * Any扩展。获得类的泛型类
 *
 * @return  类的泛型类
 */
fun Any.toInsertRow():  HashMap<String, Any?> {
    val row = HashMap<String, Any?>()
    val clz = this::class.java
    for (field in clz.getAllDeclaredFields()) {
        val anno = field.getAnnotation(Column::class.java) ?: continue
        if (!anno.insertable) {
            continue
        }

        row[anno.name] = fieldValue(this, field)
    }
    return row
} // Fun Serializable.toInsertRow()

/**
 * 转换实体到更新行
 *
 * @return  更新行
 */
fun Any.toUpdateRow(): HashMap<String, Any?> {
    val row = HashMap<String, Any?>()
    val clz = this::class.java
    for (field in clz.getAllDeclaredFields()) {
        val anno = field.getAnnotation(Column::class.java) ?: continue
        if (!anno.updatable) {
            continue
        }

        row[anno.name] = fieldValue(this, field)
    }

    return row
} // Fun toUpdateRow()

/**
 * 构建values值
 *
 * @return  values列表
 */
private fun fieldValue(obj: Any, field: Field):  Any? {
    val accessible = field.isAccessible     //canAccess(obj)
    field.isAccessible = true
    val value = field.get(obj)
    field.isAccessible = accessible
    return value
} // Fun fieldValue()
