/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.mysql

import com.sybotan.base.extensions.isBaseType
import com.sybotan.base.extensions.toJson
import com.sybotan.base.extensions.toJsonAll
import com.sybotan.database.SAbstractFunction
import com.sybotan.database.SFilter
import com.sybotan.database.enums.SDatabaseType
import com.sybotan.database.enums.SOps
import com.sybotan.mybatis.SAbstractDriver
import com.sybotan.mybatis.SSqlProvider
import org.apache.ibatis.jdbc.SQL
import org.slf4j.LoggerFactory

/**
 * MySQL驱动
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SMySqlDriver : SAbstractDriver() {
    /** 静态对象 */
    companion object {
        /** 日志记录器 */
        private val logger = LoggerFactory.getLogger(SMySqlDriver::class.java)
    } // companion object

    /**
     * 在数据库中插入实体
     *
     * @param   argsMap         注入参数
     * @return  插入SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    override fun insert(argsMap: Map<String, Any?>): String {
        val entityClass = argsMap["entityClass"] as Class<*>
        val entity = argsMap["entity"] as HashMap<String, Any?>
        val sql = object: SQL() {
            init {
                INSERT_INTO(SSqlProvider.driver.tableName(entityClass))
                for ((key, value) in entity) {
                    if (value != null) {
                        if (value is Map<*,*>) {
                            VALUES(SSqlProvider.driver.escName(key), "'${value.toJsonAll().replace("'", "''")}'")
                        } else {
                            VALUES(SSqlProvider.driver.escName(key), SSqlProvider.driver.entityValue(entity, key))
                        }
//                        if (value::class.java.isPrimitive) {
//                            VALUES(driver.escName(key), driver.entityValue(entity, key))
//                        } else {
//                            VALUES(driver.escName(key), "'${value.toJsonAll()}'")
//                        }
                    }
                }
            }
        }.toString()

        logger.debug("INSERT SQL= $sql")

        return sql
    } // Fun insert()

    /**
     * 在数据库中替换实体
     *
     * @param   argsMap         注入参数
     * @param   insertSql       插入SQL语句
     * @return  替换SQL语句
     */
    override fun replace(argsMap: Map<String, Any?>, insertSql: String): String {
        val sql = insertSql.replace("INSERT INTO", "REPLACE INTO")
        logger.debug("REPLACE SQL= $sql")
        return sql
    } // Fun replace()

    /**
     * 根据条件删除
     *
     * @param   argsMap         注入参数
     * @return  删除SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    override fun delete(argsMap: Map<String, Any?>): String {
        val entityClass = argsMap["entityClass"] as Class<*>
        // 从入参中获得过虑条件
        val filterArgs = argsMap["filterArgs"] as List<SFilter>
        // 转换后的where语句参数
        val whereArgs = argsMap["whereArgs"] as HashMap<String, Any>
        val clause = buildWhereArgList(entityClass, filterArgs, whereArgs)

        val sql = object: SQL() {
            init {
                DELETE_FROM(SSqlProvider.driver.tableName(entityClass))
                if (clause.isNotEmpty()) {
                    WHERE(clause)
                }
            }
        }.toString()

        logger.debug("SQL= $sql")
        return sql
    } // Fun delete()

    /**
     * 删除所有记录
     *
     * @return  删除SQL语句
     */
    @Suppress("UNCHECKED_CAST")
    override fun deleteAll(argsMap: Map<String, Any?>): String {
        val entityClass = argsMap["entityClass"] as Class<*>
        val sql = object: SQL() {
            init {
                DELETE_FROM(SSqlProvider.driver.tableName(entityClass))
            }
        }.toString()

        logger.debug("SQL= $sql")
        return sql
    } // Fun deleteAll()

    /**
     * 获得表名，取实体类@Table注解的名称，如果没有注解，则取实体类类名。
     *
     * @param   clz         mapper类类型
     * @return  表名
     */
    override fun tableName(clz: Class<*>): String {
        val tableAnno = clz.getAnnotation(javax.persistence.Table::class.java)
        // 如果有@Table注解，则取注解的表名；否则使用类名做为表名
        val tableName = tableAnno?.name ?: clz.simpleName
        val name = tableName.replace(".", "`.`")
        return "`$name`"
    } // Fun tableName()

    /**
     * 将 name 加上引用符号
     *
     * @param   name    名称
     * @return  加上引用符号的名称
     */
    override fun escName(name: String): String {
        return "`$name`"
    } // Fun escName()

    /**
     * 实体类的值，如果不是基本类型，则转换为json
     *
     * @param   entity      实体类
     * @param   name        名称
     * @return  加上引用符号的名称
     */
    override fun entityValue(entity: HashMap<String, Any?>, name: String): String {
        val value = entity[name] ?: return "null"
        if (value.isBaseType()) {
            return "#{entity.$name}"
        }
        return "'${value.toJson()}'"
    } // Fun entityValue()

    /**
     * 生成limit语句
     *
     * @param   offset      偏移位置
     * @param   count       取得的记录数
     * @return  LIMIT子句
     */
    override fun limit(offset: Int?, count: Int?): String {
        return " LIMIT ${offset ?: 0}, ${count ?: 1}"
    } // Fun limit()

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 函数
    /**
     * 处理函数支持
     *
     * @param   func        函数对象
     * @param   colName     字段名
     * @return  生成的SQL语句
     */
    override fun processFunction(func: SAbstractFunction, colName: String): String {
        func.colName = colName
        return func.toSql(SDatabaseType.MySQL)
    } // Fun processFunction()
//----------------------------------------------------------------------------------------------------------------------
    /**
     * 转换where字句参数
     *
     * @param   argName     参数名
     * @param   argValue    参数值
     * @param   ops         运算符
     * @return  转换后的SQL
     */
    override fun whereArg(argName: String, argValue: String, ops: SOps): String {
//        var name = obj as String
//
//        val colName: String
//        val argValue: String
//        if (path == null) {
//            colName = "`$name`"
//            argValue = "#{whereArgs.$argName}"
//        } else {
//            colName = "JSON_EXTRACT(`$name`, '\$.$path')"
//            argValue = "#{whereArgs.$argName}"
//        }

        when (ops) {
            SOps.EQ -> {     // 等于
                return "$argName = $argValue"
            }
            SOps.NE -> {     // 不等于
                return "$argName <> $argValue"
            }
            SOps.GT -> {     // 大于
                return "$argName > $argValue"
            }
            SOps.GTE -> {    // 大于等于
                return "$argName >= $argValue"
            }
            SOps.LT -> {     // 小于
                return "$argName < $argValue"
            }
            SOps.LTE -> {    // 小于等行
                return "$argName <= $argValue"
            }
            SOps.CONTAIN -> {    // 包含
                return "$argName LIKE CONCAT(\"%\" , $argValue, \"%\")"
            }
            SOps.STARTWITH -> {    // 以指定字符串开始
                return "$argName LIKE CONCAT($argValue, \"%\")"
            }
            SOps.ENDWITH -> {    // 以指定字符串结束
                return "$argName LIKE CONCAT(\"%\" , $argValue)"
            }
            SOps.ISNULL -> {     // 为空
                return "ISNULL($argName)"
            }
            else -> {
            }
        }
        return ""
    } // Fun whereArg()

    /**
     * 转换参数名SQL语句
     *
     * @param   name        字段名
     * @param   path        json路径。如果不是json对象，则该值为空
     * @return  转换后的SQL
     */
    override fun argName(name: String, path: String?): String {
        return if (path == null) {
            "`$name`"
        } else {
            "JSON_EXTRACT(`$name`, '\$.$path')"
        }
    } // Fun argName()

    /**
     * 转换参数值SQL语句
     *
     * @param   argName     参数名
     * @param   isJson      是否Json对象
     * @return  转换后的SQL
     */
    override fun argValue(argName: String, isJson: Boolean): String {
        return if (isJson) {
            "#{whereArgs.$argName}"
        } else {
            "#{whereArgs.$argName}"
        }
    } // Fun argValue()

    /**
     * 转换IN条件表达式
     *
     * @param   name        字段名
     * @param   path        json路径，如果字段不为json，则path为null
     * @param   inArg       in操作参数
     * @return  转换后的SQL
     */
    override fun inArg(name: String, path: String?, inArg: String): String {
        return if (path == null) {
            "`$name` IN ($inArg)"
        } else {
            "JSON_EXTRACT(`$name`,'\$.$path') IN ($inArg)"
        }
    } // Fun inArg()

    /**
     * 将名称及json路径转换为SQL
     *
     * @param   name        字段名
     * @param   path        json路径，如果字段不为json，则path为null
     * @return   转换后的SQL
     */
    override fun columnName(name: String, path: String?): String {
        return if (path == null) {
            "`$name`"
        } else {
            "JSON_EXTRACT(`$name`, '\$.$path')"
        }
    }

    /**
     * 将参数值转换为SQL语句
     *
     * @param   value       值
     * @param   isJson      值是否转换为json格式
     * @return  转换后的SQL
     */
    override fun argValueToJson(value: String, isJson: Boolean): String {
        return value
    } // Fun argValueToJson()

    /**
     * 更新Json类型字段
     *
     * @param   name        字段名
     * @param   value       值
     * @return  更新SQL字符串
     */
    override fun updateJson(name: String, value: Any): String {
        return "JSON_MERGE(IFNULL(`$name`,'{}'), '${value.toJsonAll()}')"
    } // Fun updateJson()


} // Class SMySqlDriver
